﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using System.Threading;
using System.IO;
using AVRBootLoaderNamespace;
using Ini;

namespace Bootloader
   {
   public partial class frmBootloader : Form
      {
      //public string sPortname = "COM1";

      private System.IO.Ports.SerialPort MySerialPort;
      private string sFilename = "";

      AVRBootLoaderNamespace.AVRBootLoader MyAVRBootloader;

      delegate void ProgrammingThreadFinishedHandler();
      private ThreadStart ts;
      private Thread t;

      public frmBootloader()
         {
         InitializeComponent();
         }

      private void cmdDatei_Click(object sender, EventArgs e)
         {
         OpenFileDialog openFileDialog = new OpenFileDialog();

         openFileDialog.CheckFileExists = true;
         openFileDialog.CheckPathExists = true;
         openFileDialog.Title = "Datei für Software Update auswählen";
         openFileDialog.Filter = "Hex Dateien (*.hex)|*.hex|Alle Dateien (*.*)|*.*";
         openFileDialog.FilterIndex = 1;
         openFileDialog.InitialDirectory = txtDateiname.Text.Trim();
         openFileDialog.FileName = txtDateiname.Text.Trim();
         openFileDialog.RestoreDirectory = true;

         if(openFileDialog.ShowDialog() == DialogResult.OK)
            txtDateiname.Text = openFileDialog.FileName;
         }

      private void frmBootloader_Load(object sender, EventArgs e)
         {
         string apppath = Application.StartupPath;
         apppath += "\\settings.cfg";
          
         IniFile ini = new IniFile(apppath);
         txtPass.Text = ini.IniReadValue("Config", "Pass");
         txtDateiname.Text = ini.IniReadValue("Config", "File");
         string cport = ini.IniReadValue("Config", "Comport");
         string baudrate = ini.IniReadValue("Config", "Baudrate");

         this.MySerialPort = new System.IO.Ports.SerialPort();

         this.MySerialPort.ReadTimeout = 2000;  // Timeout auf 2 Sekunden
         this.MySerialPort.WriteTimeout = 2000;  // Timeout auf 2 Sekunden
         this.MySerialPort.BaudRate = 115200;
         this.MySerialPort.DataBits = 8;
         this.MySerialPort.Parity = System.IO.Ports.Parity.None;
         this.MySerialPort.StopBits = System.IO.Ports.StopBits.One;
         this.MySerialPort.Handshake = System.IO.Ports.Handshake.None;

         string[] mynames = System.IO.Ports.SerialPort.GetPortNames();
         int i;
          for (i=0; i<mynames.Length; i++)
            this.selComport.Items.Add(mynames[i]);

          try
          {
            if (cport.Length == 0) { this.selComport.SelectedIndex = 0; }
            else { this.selComport.SelectedIndex = Convert.ToInt32(cport); }
          }
          catch
          {
            if (this.selComport.Items.Count > 0)
             { this.selComport.SelectedIndex = 0; }
          }
          try
          {
            if (baudrate.Length == 0) { this.selBaudrate.SelectedIndex = 0; }
            else { this.selBaudrate.SelectedIndex = Convert.ToInt32(baudrate); }
          }
          catch
          {
            if (this.selBaudrate.Items.Count > 0)
             { this.selBaudrate.SelectedIndex = 0; }
          }

         MyAVRBootloader = new AVRBootLoader(this.MySerialPort);
         
         }

      private void frmBootloader_Closing(object sender, System.ComponentModel.CancelEventArgs e)
      {
          string apppath = Application.StartupPath;
          apppath += "\\settings.cfg";

          IniFile ini = new IniFile(apppath);
          ini.IniWriteValue("Config", "Pass", txtPass.Text);
          ini.IniWriteValue("Config", "File", txtDateiname.Text);
          ini.IniWriteValue("Config", "Comport", Convert.ToString(this.selComport.SelectedIndex));
          ini.IniWriteValue("Config", "Baudrate", Convert.ToString(this.selBaudrate.SelectedIndex));
      }

      private void cmdOK_Click(object sender, EventArgs e)
         {
      
          this.sFilename = txtDateiname.Text.Trim();

         
         // Gibt es die Datei
         if(!System.IO.File.Exists(this.sFilename))
            {
            Util.MyUtils.MessageInfo(String.Format("Die Datei '{0}' konnte nicht gefunden werden!", this.sFilename));
            txtDateiname.Focus();
            return;
            }

         this.Cursor = Cursors.WaitCursor;
         // Com-Port öffnen
         if(!this.MySerialPort.IsOpen)
            {
            try
               {
                this.MySerialPort.PortName = this.selComport.Text;
                this.MySerialPort.BaudRate = Convert.ToInt32(this.selBaudrate.Text);
                //this.MySerialPort.PortName = "COM8";
                this.MySerialPort.Open();
               }
            catch
               {
               this.Cursor = Cursors.Default;
               Util.MyUtils.MessageError(String.Format("Fehler beim Öffnen des Com-Ports '{0}'!", this.MySerialPort.PortName));
               return;
               }
            }

         // Jetzt geht´s los
         this.txtDateiname.Enabled = false;
         this.cmdDatei.Enabled = false;
         this.cmdCancel.Visible = true;
         this.cmdOK.Visible= false;
         this.cmdSchliessen.Enabled = false;

         this.MyAVRBootloader.Password = txtPass.Text;

         this.ProgressProgrammieren(0);
         this.ProgressUeberpruefen(0);

         Util.MyUtils.MessageInfo("Bitte trennen Sie jetzt das Gerät von der Stromversorgung!\n"+
            "Klicken Sie dann auf OK und schließen danach das Gerät wieder an die Stromversorgung an!");

         this.ts = new ThreadStart(this.ProgrammingThread);
          
         this.t = new Thread(ts);

         this.t.Start();

         return;
         }

      private void cmdCancel_Click(object sender, EventArgs e)
         {
         if(this.MyAVRBootloader.Busy)
            {
            if(Util.MyUtils.MessageQuestion("Sind Sie sicher, dass Sie den laufenden Vorgang abbrechen möchten?\n" +
               "Dies könnte zu einem nicht funktionsfähigem Gerät führen!") == DialogResult.No)
               return;
            if(this.MyAVRBootloader.Busy)
               {
               this.MyAVRBootloader.AVRBootloaderAbort();
               while(this.MyAVRBootloader.Busy)
                  Thread.Sleep(100);
               return;
               }
            }
         return;
         }

      private void ProgrammingThread()
         {
         AVRBootLoader.ProgressChangedEventHandler ProgressHandler;
         
         // Verbinden
         this.MyAVRBootloader.AVRBootloaderConnect();
         while(this.MyAVRBootloader.Busy)
            Thread.Sleep(100);

         if(this.MyAVRBootloader.Aborted)
            {
            this.ProgrammingThreadFinished();
            return;
            }
         if(!this.MyAVRBootloader.Success)
            {
            Util.MyUtils.MessageError("Beim Verbinden ist ein Fehler aufgetreten!\n'" + this.MyAVRBootloader.ErrorMessage+"'");
            this.ProgrammingThreadFinished();
            return;
            }

         // Programmieren
         // Progress-Event anmelden
         ProgressHandler = this.ProgressProgrammieren;
         this.MyAVRBootloader.ProgressChanged += ProgressHandler;

         this.MyAVRBootloader.AvrBootloaderProgram(sFilename);
         while(this.MyAVRBootloader.Busy)
            Thread.Sleep(100);

         if(this.MyAVRBootloader.Aborted)
            {
            this.MyAVRBootloader.ProgressChanged -= ProgressHandler;
            this.ProgrammingThreadFinished();
            return;
            }
         if(!this.MyAVRBootloader.Success)
            {
            this.MyAVRBootloader.ProgressChanged -= ProgressHandler;
            Util.MyUtils.MessageError("Beim Programmieren ist ein Fehler aufgetreten!\n" +
               "ACHTUNG: Das Gerät könnte in einem nicht funktionsfähigen Zustand sein!");
               this.ProgrammingThreadFinished();
               return;
            }
         this.MyAVRBootloader.ProgressChanged -= ProgressHandler;

         // Überprüfen
         // Progress-Event anmelden
         ProgressHandler = this.ProgressUeberpruefen;
         this.MyAVRBootloader.ProgressChanged += ProgressHandler;

         this.MyAVRBootloader.AvrBootloaderVerify(sFilename);
         while(this.MyAVRBootloader.Busy)
            Thread.Sleep(100);

         if(this.MyAVRBootloader.Aborted)
            {
            this.MyAVRBootloader.ProgressChanged -= ProgressHandler;
            this.ProgrammingThreadFinished();
            return;
            }
         if(!this.MyAVRBootloader.Success)
            {
            this.MyAVRBootloader.ProgressChanged -= ProgressHandler;
            Util.MyUtils.MessageError("Beim Überprüfen ist ein Fehler aufgetreten!\n" +
               "ACHTUNG: Das Gerät könnte in einem nicht funktionsfähigem Zustand sein!");
            this.ProgrammingThreadFinished();
            return;
            }
         this.MyAVRBootloader.ProgressChanged -= ProgressHandler;

         // Trennen
         this.MyAVRBootloader.AVRBootloaderDisconnect();
         while(this.MyAVRBootloader.Busy)
            Thread.Sleep(100);

         if(this.MyAVRBootloader.Aborted)
            {
            this.ProgrammingThreadFinished();
            return;
            }
         if(!this.MyAVRBootloader.Success)
            {
            Util.MyUtils.MessageError("Beim Trennen ist ein Fehler aufgetreten!");
            this.ProgrammingThreadFinished();
            return;
            }
         this.ProgrammingThreadFinished();
         }

      private void ProgrammingThreadFinished()
         {
         if(this.InvokeRequired)
            {
            this.Invoke(new ProgrammingThreadFinishedHandler(this.ProgrammingThreadFinished));
            }
         else
            {
            this.txtDateiname.Enabled = true;
            this.cmdDatei.Enabled = true;
            this.cmdOK.Visible = true;
            this.cmdCancel.Visible = false;
            this.cmdSchliessen.Enabled = true;
            this.Cursor = Cursors.Default;

            if (this.MyAVRBootloader.Success)
            {
                if (this.MySerialPort.IsOpen) this.MySerialPort.Close();
                Util.MyUtils.MessageInfo(String.Format("{0} Bytes erfolgreich geschrieben!", this.MyAVRBootloader.BytesWritten));
            }
            }
         if (this.MySerialPort.IsOpen)
             this.MySerialPort.Close();
         }

      private void ProgressProgrammieren(int nProgress)
         {
         AVRBootLoader.ProgressChangedEventHandler handler;

         if(this.proProgrammieren.InvokeRequired)
            {
            handler = new AVRBootLoader.ProgressChangedEventHandler(this.ProgressProgrammieren);
            object[] args = { nProgress };
            this.proProgrammieren.BeginInvoke(handler, args);
            }
         else
            {
            lblProgrammieren.Text = "Laden " + nProgress.ToString() + "%";
            this.proProgrammieren.Value = nProgress;
            this.proProgrammieren.Refresh();
            }
         }

      private void ProgressUeberpruefen(int nProgress)
         {
         AVRBootLoader.ProgressChangedEventHandler handler;

         if(this.proUeberpruefen.InvokeRequired)
            {
            handler = new AVRBootLoader.ProgressChangedEventHandler(this.ProgressUeberpruefen);
            object[] args = { nProgress };
            this.proUeberpruefen.BeginInvoke(handler, args);
            }
         else
            {
            lblUeberpruefen.Text = "Prüfen " + nProgress.ToString() + "%";
            this.proUeberpruefen.Value = nProgress;
            this.proUeberpruefen.Refresh();
            }
         }

      private void cmdSchliessen_Click(object sender, EventArgs e)
         {
         if(this.MySerialPort.IsOpen)
            this.MySerialPort.Close();
         this.Close();
         }
      }
   }
